#pragma once
#include <J3ML/LinearAlgebra/Vector4.hpp>
#include <Colors.hpp>
namespace JGL::J2D {
    class Light;
    class PointLight;
}

// TODO Static member function to transform to RenderTarget space.
// For ex, If the render target would be positioned at 10, 10 and the light is at 0, 0. Then fix it such that for the Render Target the light is at -10, -10.
/// Only works if we know where the Render Target will be on screen beforehand. But that's the only scenario you'd need to be able to do this anyways - Redacted.
class JGL::J2D::Light {
protected:
    Vector4 light_pos = Vector4::Zero;
    Color4 ambient = Colors::White;
    Color4 diffuse = Colors::White;
    float constant_attenuation = 1;
    float linear_attenuation = 0;
    float quadratic_attenuation = 0;
public:
    /// @returns The position of this light in screen-world 2D space.
    [[nodiscard]] Vector2 Position() const;
    /// @returns The ambient light color.
    [[nodiscard]] Color4 Ambient() const;
    /// @returns The diffuse light color.
    [[nodiscard]] Color4 Diffuse() const;
    /// @returns The constant attenuation value
    [[nodiscard]] float Constant() const;
    /// @returns The linear attenuation value.
    [[nodiscard]] float Linear() const;
    /// @returns The quadratic attenuation value.
    [[nodiscard]] float Quadratic() const;
public:
    /// Set the position of this light in screen-world 2D space.
    void Position(const Vector2& new_position);
    /// Set the ambient light color.
    void Ambient(const Color4& new_ambient);
    /// Set the diffuse light color.
    void Diffuse(const Color4& new_diffuse);
    /// Set the constant attenuation value (0 - 1).
    void Constant(float new_constant_attenuation);
    /// Set the linear attenuation value (0 -1).
    void Linear(float new_linear_attenuation);
    /// Set the quadratic attenuation value (0 - 1).
    void Quadratic(float new_quadratic_attenuation);
public:
    /// @returns float (0 - 1) representing how much this light impacts a given position in 2D space.
    [[nodiscard]] virtual float MeasureIntensityAtPosition(const Vector2& position) const = 0;
};

class JGL::J2D::PointLight : public Light {
public:
    [[nodiscard]] float MeasureIntensityAtPosition(const Vector2& position) const override;
public:
    PointLight(const Vector2& position, const Color4& ambient, const Color4& diffuse,
    float constant_attenuation = 1, float linear_attenuation = 0, float quadratic_attenuation = 0);
};

